@@ -4,7 +4,6 @@ import io
 import os
 import sys
 import datetime
-import time
 import traceback
 import re
 from os import path
@@ -70,6 +69,16 @@ def testcase_name(test_method):
     return result
 
 
+def resolve_filename(filename):
+    # Try to make filename relative to current directory.
+    try:
+        rel_filename = os.path.relpath(filename)
+    except ValueError:
+        return filename
+    # if not inside folder, keep as-is
+    return filename if rel_filename.startswith('../') else rel_filename
+
+
 class _DuplicateWriter(io.TextIOBase):
     """
     Duplicate output from the first handle to the second handle
@@ -517,7 +526,11 @@ class _XMLTestResult(_TextTestResult):
         """
         Returns the test method name.
         """
-        return test_id.split('.')[-1]
+        # Trick subtest referencing objects
+        subtest_parts = test_id.split(' ')
+        test_method_name = subtest_parts[0].split('.')[-1]
+        subtest_method_name = [test_method_name] + subtest_parts[1:]
+        return ' '.join(subtest_method_name)
 
     _test_method_name = staticmethod(_test_method_name)
 
@@ -543,7 +556,10 @@ class _XMLTestResult(_TextTestResult):
         xml_testsuite.appendChild(testcase)
 
         class_name = re.sub(r'^__main__.', '', test_result.id())
-        class_name = class_name.rpartition('.')[0]
+
+        # Trick subtest referencing objects
+        class_name = class_name.split(' ')[0].rpartition('.')[0]
+
         testcase.setAttribute('classname', class_name)
         testcase.setAttribute(
             'name', _XMLTestResult._test_method_name(test_result.test_id)
@@ -553,8 +569,7 @@ class _XMLTestResult(_TextTestResult):
 
         if test_result.filename is not None:
             # Try to make filename relative to current directory.
-            filename = os.path.relpath(test_result.filename)
-            filename = test_result.filename if filename.startswith('../') else filename
+            filename = resolve_filename(test_result.filename)
             testcase.setAttribute('file', filename)
 
         if test_result.lineno is not None:
@@ -585,13 +600,13 @@ class _XMLTestResult(_TextTestResult):
                 _XMLTestResult._createCDATAsections(
                     xml_document, result_elem, error_info)
 
-        if test_result.stdout is not None:
+        if test_result.stdout:
             systemout = xml_document.createElement('system-out')
             testcase.appendChild(systemout)
             _XMLTestResult._createCDATAsections(
                 xml_document, systemout, test_result.stdout)
 
-        if test_result.stderr is not None:
+        if test_result.stderr:
             systemout = xml_document.createElement('system-err')
             testcase.appendChild(systemout)
             _XMLTestResult._createCDATAsections(
